# Woden

Woden is a wrapper around multiple wanderly search engines in development. 

So far, woden is capable of searching for profiles within Google Scholar, provided any range of parameters. The information that will be gathered is:

- Name: The name of the author
- Citations: The number of citations an author has
- Labels: The associated labels (ex. Ecology, Mathematics, Sociology)
- Query: The query string that leads to an authors Google Scholar profile
- User: The unique user identifier
- Papers: The papers that are associated with the authors Google Scholar profile
  - Given Google Scholar's methodology to obtain this information, there is a margin of error. Some papers list in a profile may be of another author with the same name.

The information on papers that is gathered is:

- Title: The title of a paper
- Citations: The number of times a papers has been cited
- Publication Year: The year the paper was published

## Getting Started

You can start using woden by importing the `engine` module, and calling on the `GoogleScholar()` constructor.

```{python}
from woden import engine

wander = engine.GoogleScholar()
```

### Searching for Profiles

To search for profiles, you can call on the `search_profile` method to find Google Scholar profiles that fit a passed parameters.

```{python}
# Searching for profiles with:
# . The name "John Smith"
# . Involved in the field of History
profiles: List[Profile] = wander.search_profile("John Smith", "label:History")
```

```python
# Searching for profiles of individuals in:
# . Harvard University
# . Involved in the field of Physics
profiles: List[Profile] = wander.search_profile("Harvard University", "label:Physics")
```

If you are interested in knowing who is the most or least cited indvidual with your passes parameters, you can use the `cited` argument.

```{python}
profile: Profile = wander.search_profile("London School of Economics", cited="most")
```

```{python}
profile: Profile = wander.search_profile("London School of Economics", cited="least")
```

The `search_profiles` method will return a list of Profile by default, unless you have specified that you want the most or least cited Profile. You can also ask it simplify the result by returning the first Profile in the list of profiles.

```{python}
profile: Profile = wander.search_profile("University of British Columbia", simpify=True)
```

You may wish to simplify the output if you know that the search will only have one result, or if you are only interested in the first profile.

### Already know someones User ID?

If you already know someones Google Scholars unique identifier, and wish to collect their profiles data, you should use the `search_by_user_id` method.

```{python}
profile: Profile = wander.search_by_user_id("<profile-unique-identifier>")
```

### Collecting a Profile's Papers

It may be useful to also have a list of all the papers that are associated with an individuals profile. As mentioned earlier, the papers in a profile are not 100% accurate. To obtain an individuals papers, you can call the `search_papers` method in the `Profile` class.

```{python}
profile.search_papers()
```

The call to this method will modify the profile object and fill the `papers` field. If you wish to store a profile's papers as a separate variable, you can pass the `return_papers` argument.

```{python}
papers: Generator[Paper, None, None] = profile.search_papers(return_papers=True)
```

Since any given profile may hold a bunch of papers, the return type is of generator, which takes up less memory because of its lazy evaluation.

__NOTE:__ Not all papers have publication or citations in a Profile.

## Saving your work

You can save the results of a profile by calling the `to_json` method on a profile object. You must pass a filename with the JSON extension.

```{python}
profile: Profile = wander.search_profile("label:Biodiversity", "Oxford", cited="most")

profile.to_json("file.json")
```

## TODO:

- Priority:
  - Pass arbitrary number of labels
  - Differentiate profiles with the same name by labels
- Additional:
  - Integrate other wanderly search enginers (ex. NSERC, PubMed, Web of Science)
  - Obtain h-index associated with profiles of individuals in different platforms
  - Obtain co-authors from a profile

