import re

from ..model import ScholarProfile
from ..utils import select, ignore, select_href
from bs4 import BeautifulSoup
from typing import List

EMAIL_DOMAIN = re.compile(r'Verified email at (.*)')


def scholar_profile_loader(profile_soup: BeautifulSoup,
                           meta) -> ScholarProfile:
    '''Parses a BeautifulSoup object and returns a ScholarProfile object'''
    name: str = select(profile_soup, '#gsc_prf_in', one=True)
    affiliation: str = select(profile_soup, '.gsc_prf_ila', one=True)
    num_citations: int = int(select(profile_soup, '.gsc_rsb_std', one=True))

    email_domain: str = select_email_domain(profile_soup)

    tags: List[str] = select(profile_soup, '.gsc_prf_inta')
    homepage: str = select_href(profile_soup, '#gsc_prf_ivh')

    return ScholarProfile(name=name,
                          affiliation=affiliation,
                          num_citations=num_citations,
                          email_domain=email_domain,
                          tags=tags,
                          homepage=homepage,
                          ref=meta['ref'])


@ignore(AttributeError, TypeError)
def select_email_domain(soup):
    email = soup.select_one('#gsc_prf_ivh').find(text=True, recursive=False)
    return EMAIL_DOMAIN.match(email).group(1)
