from functools import wraps
from typing import (List, Tuple, Any, Dict, Union)
from typing import NamedTuple
from types import FunctionType


def ignore(*exceptions) -> FunctionType:
    def inner(func: FunctionType) -> FunctionType:
        @wraps(func)
        def wrapper(*args: Tuple[Any],
                    **kwargs: Dict[Any, Any]) -> Union[Any, None]:
            try:
                result: Any = func(*args, **kwargs)
            except exceptions as err:
                return None
            else:
                return result

        return wrapper

    return inner


@ignore(AttributeError, TypeError)
def select(html: 'bs4.BeautifulSoup', css: str,
           one: bool = False) -> List[str]:
    '''Parses a BeautifulSoup object with given css paramater, text is lower case.'''
    if one:
        return lower_txt(html.select_one(css))
    else:
        return [lower_txt(node) for node in html.select(css)]


@ignore(AttributeError, TypeError)
def select_href(html: 'bs4.BeautifulSoup', css: str, one: bool):
    if one:
        return html.select_one(css).a['href']
    else:
        return [node.a['href'] for node in html.select(css)]


def lower_txt(soup: 'bs4.BeautifulSoup') -> str:
    '''BeautifulSoup object has text extracted and put in lower case '''
    return soup.get_text().lower()
